
' chessboard.bi

#include "fbgfx.bi"
#include "colors.bi"
#include "pieces.bi"
#include "cursor.bi"

const BRDX = 0
const BRDY = 0
const SQRW = 48
const BRDW = 8 * SQRW
const BUFW = 10 * SQRW
const EMPTY = 0
const PAWN = 1
const KING = 6
const BLACK = -1
const BACKGROUND = 0
const DARKSQUARE = 7
const GRAB = 1
const GRABBING = 2
const CURW = 32
const UPSIDEDOWN = false

PiecePlacement:

data -2,-3,-4,-5,-6,-4,-3,-2
data -1,-1,-1,-1,-1,-1,-1,-1
data  0, 0, 0, 0, 0, 0, 0, 0
data  0, 0, 0, 0, 0, 0, 0, 0
data  0, 0, 0, 0, 0, 0, 0, 0
data  0, 0, 0, 0, 0, 0, 0, 0
data  1, 1, 1, 1, 1, 1, 1, 1
data  2, 3, 4, 5, 6, 4, 3, 2

type TGraphChessboard
  as integer board(1 to 8, 1 to 8)
  as fb.image ptr images(BLACK * KING to DARKSQUARE)
  as fb.image ptr cursor(BACKGROUND to GRABBING)
  as fb.image ptr buffer
  as uinteger LSC, DSC
  as boolean SLASHFILL
  as integer mx, my, mb, ox = 0, oy = 0, cur = GRAB
  declare constructor (aLSC as const uinteger, aDSC as const uinteger, aSLASHFILL as const boolean)
  declare sub CreateImages()
  declare sub DestroyImages()
  declare sub DrawImages()
  declare sub PiecesOutline()
  declare sub SaveImages()
  declare sub LoadBoard()
  declare sub Redraw()
  declare sub BufferToScreen()
  declare sub RedrawBackgroundImage(aX as const integer, aY as const integer)
  declare function XToBoard() as integer
  declare function YToBoard() as integer
  declare function XYToName(aX as const integer, aY as const integer) as string
  declare sub PreventCursorExit()
  declare sub OnStartGrabbing(byref dx as integer, byref dy as integer, byref px as integer, byref py as integer, byref brd as integer)
  declare sub OnStopGrabbing(byref px as integer, byref py as integer, brd as const integer)
  declare function MouseMoved() as boolean
  declare sub DrawPiece(dx as const integer, dy as const integer, brd as const integer)
  declare sub RestoreCursorBackground()
  declare sub DrawCursor()
  declare sub SaveMousePos()
  declare function IsMouseOnPiece() as boolean
end type

constructor TGraphChessboard (aLSC as const uinteger, aDSC as const uinteger, aSLASHFILL as const boolean)
  LSC = aLSC
  DSC = aDSC
  SLASHFILL = aSLASHFILL
  CreateImages()
  DrawImages()
  LoadBoard()
  Redraw()
end constructor

sub TGraphChessboard.CreateImages()
  buffer = imagecreate(BUFW, BUFW, LSC)
  for i as integer = BLACK * KING to DARKSQUARE
    images(i) = imagecreate(SQRW, SQRW)
  next i
  for i as integer = BACKGROUND to GRABBING
    cursor(i) = imagecreate(CURW, CURW)
  next i
end sub

sub TGraphChessboard.DestroyImages()
  imagedestroy(buffer)
  for i as integer = BLACK * KING to DARKSQUARE
    imagedestroy(images(i))
  next i
  for i as integer = BACKGROUND to GRABBING
    imagedestroy(cursor(i))
  next i
end sub

sub TGraphChessboard.DrawImages()
  dim datum as string
  dim as uinteger c1, c2
  
  restore WarlordChessPieces
  for i as integer = PAWN to KING
    for y as integer = 0 to SQRW - 1
      read(datum)
      for x as integer = 0 to SQRW - 1
        select case as const datum[x]
          case asc("0")
            c1 = colors.MAGENTA
            c2 = colors.MAGENTA
          case asc("1")
            c1 = colors.GRAY
            c2 = colors.GRAY
          case asc("2")
            c1 = colors.IVORY
            c2 = colors.BLACK
        end select
        pset images( i), (x, y), c1
        pset images(-i), (x, y), c2
      next x
    next y
  next i

  if SLASHFILL then
    PiecesOutline()
    for x as integer = 0 to SQRW - 1
      for y as integer = 0 to SQRW - 1
        if (x + y) mod 5 = 2 then
          pset images(DARKSQUARE), (x, y), colors.BLACK
        end if
      next y
    next x
  else
    line images(DARKSQUARE), (0, 0)-(SQRW - 1, SQRW - 1), DSC, BF
  end if

  restore HandCursor:
  for i as integer = GRAB to GRABBING
    for y as integer = 0 to CURW - 1
      read datum
      for x as integer = 0 to CURW - 1
        select case as const datum[x]
          case asc("0")
            pset cursor(i), (x, y), colors.MAGENTA
          case asc("1")
            pset cursor(i), (x, y), colors.BLACK
          case asc("2")
            pset cursor(i), (x, y), &hFFEFFF
        end select
      next x
    next y
  next i
end sub

sub TGraphChessboard.PiecesOutline()
  const WHITE = 1
  dim img as fb.image ptr = 0
  for colour as integer = BLACK to WHITE step 2
    for piece as integer = PAWN to KING
      img = images(colour * piece)
        for x as integer = 1 to SQRW - 2
          for y as integer = 1 to SQRW - 2
            if point(x, y, img) = colors.MAGENTA then
              if (point(x,     y - 1, img) = colors.GRAY) _
              or (point(x + 1, y - 1, img) = colors.GRAY) _
              or (point(x + 1, y,     img) = colors.GRAY) _
              or (point(x + 1, y + 1, img) = colors.GRAY) _
              or (point(x,     y + 1, img) = colors.GRAY) _
              or (point(x - 1, y + 1, img) = colors.GRAY) _
              or (point(x - 1, y,     img) = colors.GRAY) _
              or (point(x - 1, y - 1, img) = colors.GRAY) then
                pset img, (x, y), LSC
              end if
            end if
          next y
        next x
    next piece
  next colour
end sub

sub TGraphChessboard.SaveImages()
  dim img as fb.image ptr = imagecreate(6 * SQRW, 2 * SQRW)
  
  for i as integer = PAWN to KING
    put img, (SQRW * (i - 1), 0), images(i), pset
    put img, (SQRW * (i - 1), SQRW), images(-i), pset
  next i
  bsave("warlord-chess-graphics.bmp", img)
  imagedestroy(img)
  
  img = imagecreate(2 * CURW, CURW)
  for i as integer = GRAB to GRABBING
    put img, (SQRW * (i - 1), 0), cursor(i), pset
  next i
  bsave("hand-cursor.bmp", img)
  imagedestroy(img)
  
  img = imagecreate(SQRW, SQRW)
  put img, (0, 0), images(DARKSQUARE), pset
  bsave("slashfill.bmp", img)
  imagedestroy(img)
end sub

sub TGraphChessboard.LoadBoard()
  restore PiecePlacement:
  for y as integer = 8 to 1 step -1
    for x as integer = 1 to 8
      read(board(x, y))
    next x
  next y
end sub

sub TGraphChessboard.Redraw()
  dim as integer xToBuf, yToBuf
  line buffer, (0, 0)-(BUFW, BUFW), LSC, BF
  for y as integer = 1 to 8
    for x as integer = 1 to 8
      xToBuf = x * SQRW
      yToBuf = iif(UPSIDEDOWN, y, 9 - y) * SQRW
      if (x + y) mod 2 = 0 then
        put buffer, (xToBuf, yToBuf), images(DARKSQUARE), trans
      end if
      if board(x, y) <> EMPTY then
        put buffer, (xToBuf, yToBuf), images(board(x, y)), trans
      end if
    next x
  next y
  if (mx <> 0) or (my <> 0) then
    get buffer, (mx + SQRW - 16, my + SQRW - 16)-(mx + SQRW + 15, my + SQRW + 15), cursor(BACKGROUND)
    put buffer, (mx + SQRW - 16, my + SQRW - 16), cursor(cur), trans
  end if
end sub

sub TGraphChessboard.BufferToScreen()
  put (BRDX, BRDY), buffer, (SQRW, SQRW)-(9 * SQRW - 1, 9 * SQRW - 1), pset
end sub

sub TGraphChessboard.RedrawBackgroundImage(aX as const integer, aY as const integer)
  if SLASHFILL then
    line images(BACKGROUND), (0, 0)-(SQRW - 1, SQRW - 1), LSC, BF
    if (aX + aY) mod 2 = 0 then
      put images(BACKGROUND), (0, 0), images(DARKSQUARE), (0, 0)-(SQRW - 1, SQRW - 1), trans
    end if
  else
    line images(BACKGROUND), (0, 0)-(SQRW - 1, SQRW - 1), iif((aX + aY) mod 2 = 1, LSC, DSC), BF
  end if
end sub

function TGraphChessboard.XToBoard() as integer
  return mx \ SQRW + 1
end function

function TGraphChessboard.YToBoard() as integer
  return iif(UPSIDEDOWN, my \ SQRW + 1, 8 - my \ SQRW)
end function

function TGraphChessboard.XYToName(aX as const integer, aY as const integer) as string
  return chr(asc("a") - 1 + aX, asc("1") - 1 + aY)
end function

sub TGraphChessboard.PreventCursorExit()
  const BORDER = 12
  if mx < BORDER then
    mx = BORDER
  elseif mx > BRDW - 1 - BORDER then
    mx = BRDW - 1 - BORDER
  end if
  if my < BORDER then
    my = BORDER
  elseif my > BRDW - 1 - BORDER then
    my = BRDW - 1 - BORDER
  end if
end sub

sub TGraphChessboard.OnStartGrabbing(byref dx as integer, byref dy as integer, byref px as integer, byref py as integer, byref brd as integer)
  this.cur = GRABBING
  dx = SQRW * (this.mx \ SQRW) - this.mx
  dy = SQRW * (this.my \ SQRW) - this.my
  px = this.XToBoard()
  py = this.YToBoard()
  brd = this.board(px, py)
  this.RedrawBackgroundImage(px, py)
end sub

sub TGraphChessboard.OnStopGrabbing(byref px as integer, byref py as integer, brd as const integer)
  this.cur = GRAB
  this.board(px, py) = EMPTY
  px = this.XToBoard()
  py = this.YToBoard()
  this.board(px, py) = brd
  this.Redraw()
  this.BufferToScreen()
end sub

function TGraphChessboard.MouseMoved() as boolean
  return cbool((this.mx <> this.ox) or (this.my <> this.oy))
end function

sub TGraphChessboard.DrawPiece(dx as const integer, dy as const integer, brd as const integer)
  put this.buffer, (this.ox + dx + SQRW, this.oy + dy + SQRW), this.images(BACKGROUND), pset
  get this.buffer, (this.mx + dx + SQRW, this.my + dy + SQRW)-(this.mx + dx + 2 * SQRW - 1, this.my + dy + 2 * SQRW - 1), this.images(BACKGROUND)
  put this.buffer, (this.mx + dx + SQRW, this.my + dy + SQRW), this.images(brd), trans
end sub

sub TGraphChessboard.RestoreCursorBackground()
  if (this.ox <> 0) and (this.oy <> 0) then
    put this.buffer, (this.ox + SQRW - 16, this.oy + SQRW - 16), this.cursor(BACKGROUND), pset
  end if
end sub

sub TGraphChessboard.DrawCursor()
  get this.buffer, (this.mx + SQRW - 16, this.my + SQRW - 16)-(this.mx + SQRW + 15, this.my + SQRW + 15), this.cursor(BACKGROUND)
  put this.buffer, (this.mx + SQRW - 16, this.my + SQRW - 16), this.cursor(this.cur), trans
end sub

sub TGraphChessboard.SaveMousePos()
  this.ox = this.mx
  this.oy = this.my
end sub

function TGraphChessboard.IsMouseOnPiece() as boolean
  return cbool(this.board(this.XToBoard(), this.YToBoard()) <> EMPTY)
end function
